      program datsegment1C

      parameter (max=99999999)
      character*14 par
      character*80 file1,fileout
      real s1(max),rms(max),rstd(max),tl(max)
      real dtime1,dtime2,dt
      logical lt1,lrel,lbug

      lbug=.true.

c     number of arguments: narg
      narg=iargc()
      write(*,*)
      if (narg.ne.8) then
       if (narg.ge.1) write(*,*)' Program expects 8 arguments.'
       write(6,*)' '
       write(6,*)'Compute variability of RMS in moving data windows. '
       write(6,*)'Use this variability to identify segments with '
       write(6,*)'less glitches, data problems or even outlying events.'
       write(6,*)' '
       write(6,*)'USAGE: datsegment1C sac-file parameters'
       write(6,*)' '
       write(6,*)'parameters: window1_length t1_interval' 
       write(6,*)'            window2_length t2_interval '
       write(6,*)'            window3_length '
       write(6,*)'            max_RMS_variability abs/rel'
       write(6,*)' '
       write(6,*)'window1: Window for RMS determination (units: s).'
       write(*,*)'t1_interval: Time-step interval (units: s).'
       write(*,*)'window2: Window to determine RMS variability. '
       write(*,*)'         Use window1 < window2 (units: s).'
       write(*,*)'t2_interval: Time-step interval (units: s).'
       write(*,*)'window3: Required minimum length of data segement'
       write(*,*)'         to be extracted (unit: s).'
       write(*,*)'         Extract if: segment > window3_length &'
       write(*,*)'         RMS_variability < max_RMS_variability'
       write(*,*)'max_RMS_variability: Threshold, maximum permitted'
       write(*,*)'         standard deviation of RMS.'
       write(*,*)'abs/rel: Use either abs or rel. abs stands for'
       write(*,*)'         absolute RMS variability and rel for'
       write(*,*)'         relative RMS variability.'
       write(*,*)
       write(6,*)'Output:  (1) Each segment is output as a separate'
       write(6,*)'         sac file. A segment number is appended to'
       write(*,*)'         input filename.'
       write(*,*)'         (2) RMS: _rms is appended to input filename.'
       write(*,*)'         (3) RMS variability: _rstd is appended to'//
     & ' input filename.'
       write(6,*)' '
       write(6,*)'*** Example run: *** '
       write(*,*)'datsegment1C 2019.312.00.TEST.BHZ.sac '//
     & '5 0.1 20 1 300 .1 rel '
       write(*,*)
       write(*,*)'*** Outputs: ***'
       write(*,*)'2019.312.00.TEST.BHZ.sac_10001'
       write(*,*)'2019.312.00.TEST.BHZ.sac_10002'
       write(*,*)'...'
       write(*,*)'2019.312.00.TEST.BHZ.sac_rms'
       write(*,*)'2019.312.00.TEST.BHZ.sac_rstd'
       write(6,*)' '
       write(6,*)'IMPORTANT NOTES: '
       write(6,*)'(1) RMS is being used! Thus de-mean and/or filter the'
       write(6,*)'data before using this code.'
       write(6,*)'(2) rel refers to variance not standard deviation.'
       write(*,*)'(3) Use parameters in the order presented in the '
       write(*,*)'parameter list.'
       write(*,*)'(4) If SAC trace amplitudes are too small (often the'
       write(*,*)'case depending on units) then I multiply the '
       write(*,*)'amplitudes  by 10e+10 to avoid working with very '
       write(*,*)'small numbers.'
       write(*,*)'(5) Program may slightly change/adapt time-steps for' 
       write(*,*)'sliding windows. Check screen output with time-steps'
       write(*,*)'in case the exact value is important.'
       write(*,*)'(6) Window1 acts on the input t-series while window2'
       write(*,*)'acts on the output of the sliding window1 analysis.'
       write(*,*)'Thus, numb of samples of the window2 analysis refers'
       write(*,*)'to the output of the window1 analysis and depends '
       write(*,*)'therefore on the window1 step interval.'
       write(6,*)' '
       write(6,*)'(07/11/2019, mschimmel@geo3bcn.csic.es) '
       stop
      endif

c minimum length (sec) of data segment to write out:
      tmin=3600.

      call getarg(1,file1)
      lfile=leng(file1) 

      call getarg(2,par)
      read(par,*)wlen1
      call getarg(3,par)
      read(par,*)dtime1
      call getarg(4,par)
      read(par,*)wlen2
      call getarg(5,par)
      read(par,*)dtime2
      call getarg(6,par)
      read(par,*)wlen3
      call getarg(7,par)
      read(par,*)rstd_max
      call getarg(8,par)
      if (par(1:3).eq."rel") then
        lrel=.true.
      else if (par(1:3).eq."abs") then 
        lrel=.false.
      else
       stop ' use either rel or abs.'
      endif

c     write(*,*)'  ... get sac trace.'
c Read SAC traces:
      call rsac1(file1,s1,nlen,beg,dt,max,nerr)
c     call getnhv('nzyear',nzyear,nerr)
c     call getnhv('nzjday',nzjday,nerr)
c     call getnhv('nzhour',nzhour,nerr)
c     call getnhv('nzmin',nzmin,nerr)
c     call getnhv('nzsec',nzsec,nerr)
c     call getkhv('kstnm',kstnm1,nerr)

c     do i=1,nlen
c       s1(i)=s1(i)*10e+10
c     enddo

      t1=beg
      lt1=.true.

c window1:
      if (dtime1.le.dt) then
        ndt1=1
        if (dtime1.eq.dt) then
          dtime1=dt
          write(*,*)'SET window1 time-step interval = dt',dt
        endif
      else
c       ndt12=nint(0.5*dtime1/dt)
c       ndt1=ndt12*2
        ndt1=nint(dtime1/dt)
        dtime1=ndt1*dt
      endif

      nwlen1=nint(0.5*wlen1/dt)
      nwlen12=nwlen1*2
      nwlen121=nwlen1*2+1

c window2:
      if (dtime2.le.dtime1) then
        ndt2=1
        dtime2=dtime1
        write(*,*)'SET window2 time-step interval = window1 interval',
     & dtime1
      else
        ndt2=nint(dtime2/dtime1)
        dtime2=ndt2*dtime1
      endif

c     ndt22=nint(0.5*dtime2/dtime1)
c     ndt2=ndt22*2
c     dtime2=ndt2*dtime1
c     if (dtime2.lt.dtime1) then
c        dtime2=dtime1
c        write(*,*)'SET window2 time-step interval = window1 interval',
c    &   dtime1
c     endif

      kwlen1=nint(0.5*wlen2/dtime1)
      kwlen12=kwlen1*2
      kwlen121=kwlen1*2+1

      if (lbug) then
      write(*,*)'window1 step interval (nr samples, s):',ndt1,ndt1*dt
      write(*,*)'window1 length (nr samples, s)       :',nwlen121,
     & nwlen12*dt
      write(*,*)'window2 step interval (nr win1, s)   :',ndt2,
     & ndt2*dtime1
      write(*,*)'window2 length (nr win1, s)          :',kwlen121,
     & kwlen12*dtime1
      write(*,*)
      endif

c window1 (RMS):
      l=0
      do i=1,nlen,ndt1
        l=l+1
        n1=i-nwlen1
        if (n1.lt.1) n1=1
        n2=n1+nwlen12
        if (n2.gt.nlen) then
           n1=nlen-nwlen12
           n2=nlen
        endif
        rsum1=0.
        do n=n1,n2
          rsum1=rsum1+s1(n)*s1(n)
        enddo
        rms(l)=sqrt(rsum1/float(nwlen121))
      enddo
      ll=l

c window2 (variability of RMS):
      l=0
      do i=1,ll,ndt2
        l=l+1
        k1=i-kwlen1
        if (k1.lt.1) k1=1
        k2=k1+kwlen12
        if (k2.gt.nlen) then
           k1=ll-kwlen12
           k2=ll
        endif
        rmean=0.
        do k=k1,k2
           rmean=rmean+rms(k)
        enddo
        kk=k2-k1
        if (kk.gt.1) rmean=rmean/float(kk)
        var=0.
        do k=k1,k2
           var=var+(rms(k)-rmean)**2
        enddo
c relative variance:
        if (lrel) then
          if (rmean.le.1.e-14) then
             write(*,*)' PROBLEM WITH MEAN RMS AMPLITUDE < 1.e-14'
             stop
          endif
          var=var/rmean**2
        endif
        rstd(l)=sqrt(var/float(kk-1))
        tl(l)=(i-1)*dtime1+beg
      enddo
      kk=l

      nn=10000
      do l=1,kk
        if (rstd(l).lt.rstd_max) then
          if (.not.lt1) then
             t1=tl(l)
             lt1=.true.
          endif
        else
          if (lt1) then
             lt1=.false.
             tlen=tl(l)-t1
             if (tlen.ge.wlen3) then
               nn=nn+1
               write(*,*)'Selected window: ',t1,t1+tlen
               n1=nint( (t1-beg)/dt ) + 1
               nlen1=nint( tlen/dt ) - 2
               write(par(1:6),33)nn
  33           format("_",i5)
               fileout=file1(1:lfile)//par(1:6)
               write(*,*)fileout
               call setfhv('b',t1,nerr)
               call setnhv('npts',nlen1,nerr)
               call wsac0(fileout(1:lfile+6),dum,s1(n1),nerr)
             endif
          endif
        endif
      enddo

      call newhdr
      call setfhv('delta',dtime1,nerr)
      call setfhv('b',beg,nerr)
      call setnhv('npts',ll,nerr)
      fileout=file1(1:lfile)//"_rms"
      call wsac0(fileout(1:lfile+4),dum,rms,nerr)

      call setnhv('npts',kk,nerr)
      call setfhv('delta',dtime2,nerr)
      fileout=file1(1:lfile)//"_rstd"
      call wsac0(fileout(1:lfile+5),dum,rstd,nerr)

      end
c---------------------------------------------------------------
      integer function leng(char)
c
c M. Schimmel (schimmel@ictja.csic.es)
c
c     leng is the position of the last non blank character
c     in the character variable char
      character char*(*)
      integer*4 l
      l=len(char)
      do leng=l,1,-1
      if (char(leng:leng) .ne. ' ') return
      enddo
      leng=0
      return
      end
c-----------------------------------------------------------

